#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    sizeMatrix = 0;
    ui->setupUi(this);

    setWindowTitle("M.A.L.0. algebra calculator");

    //set the pointer to zero.
    firstMatrix = 0;
    secondMatrix = 0;
    thirdMatrix = 0;
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::resizeColumnsRows()
{
    //resize all the columns and rows of every matrixes
    ui->tableWidget->resizeColumnsToContents();
    ui->tableWidget_2->resizeColumnsToContents();
    ui->tableWidget_3->resizeColumnsToContents();
    ui->tableWidget->resizeRowsToContents();
    ui->tableWidget_2->resizeRowsToContents();
    ui->tableWidget_3->resizeRowsToContents();
}

void MainWindow::copyToTableWidget(QSMatrix *matrixQS, QTableWidget *tableWidget)
{
    //copy from matrixQS to tableWidget

    //set column and rows
    tableWidget->setColumnCount(sizeMatrix);
    tableWidget->setRowCount(sizeMatrix);

    //set the elements
    for(int row=0; row< sizeMatrix; row++)
        for(int column=0; column< sizeMatrix; column++)
        {
            //take the element
            int element = matrixQS->getElementAt(row,column);

            if(element == E_NODEP )
                tableWidget->setItem(row,column,new QTableWidgetItem(  tr( NODEP )) );
            else if(element == E_IDENTITY)
                tableWidget->setItem(row,column,new QTableWidgetItem(  tr( IDENTITY )) );
            else if(element == E_ADDED)
                tableWidget->setItem(row,column,new QTableWidgetItem(  tr( ADDED )) );
            else if(element == E_MULT)
                tableWidget->setItem(row,column,new QTableWidgetItem(  tr( MULT )) );
            else //default case, that is avoid by definition
                tableWidget->setItem(row,column,new QTableWidgetItem(  tr( NODEP )) );
        }

    //and now resize the columns and rows
    resizeColumnsRows();
}

void MainWindow::copyToLocalMatrix(QTableWidget *tableWidget, QSMatrix *matrixQS)
{
    for(int row=0; row<sizeMatrix; row++)
        for(int column=0; column<sizeMatrix; column++)
        {
            //retrieve the element
            QString element = tableWidget->item(row,column)->text();

            if(element == NODEP)
                matrixQS->setElementAt(row,column, E_NODEP);
            else if(element == IDENTITY)
                matrixQS->setElementAt(row,column, E_IDENTITY);
            else if(element == ADDED)
                matrixQS->setElementAt(row,column, E_ADDED);
            else if(element == MULT)
                matrixQS->setElementAt(row,column, E_MULT);
            else //default case, that is avoid by definition
                matrixQS->setElementAt(row,column, E_NODEP);
        }
}

void MainWindow::saveAllMatrixes()
{
    copyToLocalMatrix(ui->tableWidget, firstMatrix);
    copyToLocalMatrix(ui->tableWidget_2, secondMatrix);
    copyToLocalMatrix(ui->tableWidget_3, thirdMatrix);
}
void MainWindow::restoreAllMatrixes()
{
    copyToTableWidget(firstMatrix, ui->tableWidget);
    copyToTableWidget(secondMatrix, ui->tableWidget_2);
    copyToTableWidget(thirdMatrix, ui->tableWidget_3);
}


void MainWindow::on_actionCrete_Matrix_triggered()
{
    int returned = QInputDialog::getInteger(this, tr("Size of matrix"), tr("Size of squared matrix"), 1 ,1 );
    sizeMatrix=returned;


    //check if the object already exists
    if(firstMatrix != 0)
        firstMatrix->~QSMatrix();
    if(secondMatrix != 0)
        secondMatrix->~QSMatrix();
    if(thirdMatrix != 0)
        thirdMatrix->~QSMatrix();

    //create the matrixes
    firstMatrix = new QSMatrix(sizeMatrix);
    secondMatrix = new QSMatrix(sizeMatrix);
    thirdMatrix = new QSMatrix(sizeMatrix);

    //copy to UI
    copyToTableWidget(firstMatrix,  ui->tableWidget  );
    copyToTableWidget(secondMatrix, ui->tableWidget_2);
    copyToTableWidget(thirdMatrix,  ui->tableWidget_3);

    //then we are allowed to set enabled the buttons
    ui->tableWidget->setEnabled(true);
    ui->tableWidget_2->setEnabled(true);
    ui->tableWidget_3->setEnabled(true);

    ui->buttonplus->setEnabled(true);
    ui->buttontimes->setEnabled(true);
    ui->buttonunion->setEnabled(true);

    ui->buttoncopyfirst->setEnabled(true);
    ui->buttoncopysecond->setEnabled(true);
    ui->buttonfirstcopysecond->setEnabled(true);
    ui->buttonsecondcopyfirst->setEnabled(true);

    ui->clearbuttonone->setEnabled(true);
    ui->clearbuttontwo->setEnabled(true);
    ui->clearbuttonthree->setEnabled(true);

}

void MainWindow::on_actionUnion_Closure_triggered()
{
    standardizeMatrixes();
    saveAllMatrixes();

    //keep track of previous results
    QVector<QSMatrix*> savedMatrixes(0);

    //for safeness we put a limit
    int steplimit = 10000;
    int steps = 1;
    bool looping = false;

    //copy to second the matrix
    copyMatrixes(firstMatrix, secondMatrix);

    //execute first operation.
    QSMatrix::multiplication(firstMatrix,secondMatrix, thirdMatrix);

    //then we saved the result
    QSMatrix *result = new QSMatrix(thirdMatrix->getSize());
    copyMatrixes(thirdMatrix, result);
    savedMatrixes.append(result);

    //then iterate
    qDebug() << "Sono prima del while";

    while( (!QSMatrix::areEqual( thirdMatrix, firstMatrix)) && (steplimit > 0)  )
    {
        //then copy the result to the first matrix
        copyMatrixes(thirdMatrix, firstMatrix);

        //execute command
        QSMatrix::multiplication(firstMatrix,secondMatrix, thirdMatrix);
        //check if the result has already been founded (are we looping?)
        looping = false;
//        qDebug() << " size matrix " << savedMatrixes.size();;
        for(int i=0; i<(savedMatrixes.size()-1); i++) //we don't have to check with the last (otherwise IS the fix point)
        {
//            qDebug() << "sono alla " << i << " iterazione";
            looping = QSMatrix::areEqual(thirdMatrix, savedMatrixes.at(i));
            if(looping)
            {
//                qDebug() << "Similar to " << i << " size matrix " << savedMatrixes.size();
                i = savedMatrixes.size()+1; //we exit from the cycle
            }
        }

        if(!looping)
        {
            //otherwise we save
            QSMatrix *result = new QSMatrix(thirdMatrix->getSize());
            copyMatrixes(thirdMatrix, result);
            savedMatrixes.append(result);
        }
        else
            steplimit = 0; //we exit from the cycle

        //and then iterate

        //decrement the limit
//        qDebug() << steplimit;
        steplimit--;
        steps++;
    }

    qDebug() << "numero di iterazioni trovate " << steps;

    //take the first one
    QSMatrix tmpMatrix(thirdMatrix->getSize());

    //now we have to make unions of all the matrixes founded
    for(int i=0; i<savedMatrixes.size();i++)
    {
        QSMatrix::unions(savedMatrixes.at(i), &tmpMatrix, &tmpMatrix);
    }

    qDebug() << "ho effettuato l'unione ";

    copyMatrixes(&tmpMatrix, thirdMatrix);

    qDeleteAll(savedMatrixes);
    savedMatrixes.clear();

    //put the value on UI
    restoreAllMatrixes();
}

void MainWindow::on_actionMultiplication_Closure_triggered()
{
    standardizeMatrixes();

    //keep track of previous results
    QVector<QSMatrix*> savedMatrixes(0);


    //for safeness we put a limit
    int steplimit = 10000;
    int steps = 1;
    bool looping = false;

    saveAllMatrixes();

    //copy to second the matrix
    copyMatrixes(firstMatrix, secondMatrix);

    //execute first operation. first we erase the previous result
    QSMatrix::multiplication(firstMatrix,secondMatrix, thirdMatrix);

    //then we saved the result
    QSMatrix *result = new QSMatrix(thirdMatrix->getSize());
    copyMatrixes(thirdMatrix, result);
    savedMatrixes.append(result);

    //then iterate
    while( (!QSMatrix::areEqual( thirdMatrix, firstMatrix)) && (steplimit > 0)  )
    {
        //then copy the result to the first matrix
        copyMatrixes(thirdMatrix, firstMatrix);

        //execute command
        QSMatrix::multiplication(firstMatrix,secondMatrix, thirdMatrix);
        //check if the result has already been founded (are we looping?)
        looping = false;

        for(int i=0; i<(savedMatrixes.size()-1); i++) //we don't have to check with the last (otherwise IS the fix point)
        {

            looping = QSMatrix::areEqual(thirdMatrix, savedMatrixes.at(i));
            if(looping)
            {
                i = savedMatrixes.size()+1; //we exit from the cycle
            }
        }

        if(!looping)
        {
            //otherwise we save
            QSMatrix *result = new QSMatrix(thirdMatrix->getSize());
            copyMatrixes(thirdMatrix, result);
            savedMatrixes.append(result);
        }
        else
            steplimit = 0; //we exit from the cycle

        //and then iterate

        //decrement the limit
//        qDebug() << steplimit;
        steplimit--;
        steps ++;
    }

    qDebug() << "numero di iterazioni trovate " << steps;
    restoreAllMatrixes();

    if(looping)
    {
        QMessageBox alert;
        alert.setText("The Matrix cannot have a fix point");
        alert.exec();
    }

    qDeleteAll(savedMatrixes);
    savedMatrixes.clear();

}

void MainWindow::addMatrix()
{
    standardizeMatrixes();

    saveAllMatrixes();
    //execute operation
    QSMatrix::addition(firstMatrix,secondMatrix, thirdMatrix);
    //show it
    restoreAllMatrixes();

}

void MainWindow::unionMatrix()
{
    standardizeMatrixes();

    saveAllMatrixes();
    //execute operation
    QSMatrix::unions(firstMatrix,secondMatrix, thirdMatrix);
    //show it
    restoreAllMatrixes();

}

void MainWindow::multMatrix()
{
    standardizeMatrixes();

    saveAllMatrixes();

    //execute operation
    QSMatrix::multiplication(firstMatrix,secondMatrix,thirdMatrix);
    //show it
    restoreAllMatrixes();
}


void MainWindow::standardizeMatrixes()
{
    //    il compare funziona che da 0 se è uguale. 0 = false
    for(int row=0; row< sizeMatrix; row++)
    {
        for(int column=0; column<sizeMatrix; column++)
            if(ui->tableWidget->item(row,column)->text().compare(NODEP) )
                if(ui->tableWidget->item(row,column)->text().compare(IDENTITY) )
                    if(ui->tableWidget->item(row,column)->text().compare(ADDED) )
                        if(ui->tableWidget->item(row,column)->text().compare(MULT) )
                            ui->tableWidget->setItem(row,column,new QTableWidgetItem(tr( NODEP )) );
    }

    for(int row=0; row< sizeMatrix; row++)
    {
        for(int column=0; column<sizeMatrix; column++)
        {
            if(ui->tableWidget_2->item(row,column)->text().compare(NODEP) )
                if(ui->tableWidget_2->item(row,column)->text().compare(IDENTITY) )
                    if(ui->tableWidget_2->item(row,column)->text().compare(ADDED) )
                        if(ui->tableWidget_2->item(row,column)->text().compare(MULT) )
                            ui->tableWidget_2->setItem(row,column,new QTableWidgetItem(tr( NODEP )) );
        }
    }

}

void MainWindow::copyMatrixes(QSMatrix *from, QSMatrix *to)
{
    //copy values from "from" to "to"
    for(int row=0; row< sizeMatrix; row++)
        for(int column=0; column< sizeMatrix; column++)
            to->setElementAt(row,column, from->getElementAt(row,column));
}


void MainWindow::resetMatrix(QSMatrix *a)
{
    a->reset();
}


void MainWindow::firstToSecond()
{
    saveAllMatrixes();
    copyMatrixes(firstMatrix, secondMatrix);
    restoreAllMatrixes();
}
void MainWindow::secondToFirst()
{
    saveAllMatrixes();
    copyMatrixes(secondMatrix, firstMatrix);
    restoreAllMatrixes();
}
void MainWindow::thirdToFirst()
{
    saveAllMatrixes();
    copyMatrixes(thirdMatrix, firstMatrix);
    restoreAllMatrixes();
}
void MainWindow::thirdToSecond()
{
    saveAllMatrixes();
    copyMatrixes(thirdMatrix, secondMatrix);
    restoreAllMatrixes();
}
void MainWindow::resetOne()
{
    saveAllMatrixes();
    resetMatrix(firstMatrix);
    restoreAllMatrixes();
}
void MainWindow::resetTwo()
{
    saveAllMatrixes();
    resetMatrix(secondMatrix);
    restoreAllMatrixes();
}
void MainWindow::resetThree()
{
    saveAllMatrixes();
    resetMatrix(thirdMatrix);
    restoreAllMatrixes();
}


//parsing the program
void MainWindow::parseAndTypeProgram()
{
    //here we are testing the number of steps for M^u
    //select the size
    on_actionCrete_Matrix_triggered();

    //keep track of the max
    int maxsteps = 0;

    for(int i=0; i< ( pow(4,(sizeMatrix*sizeMatrix) )); i++)
    {
        //for safeness we put a limit
        int steplimit = 10000;
        int steps = 0;
        bool looping = false;

        //keep track of previous results
        QVector<QSMatrix*> savedMatrixes(0);

        //copy to second the matrix
        QSMatrix::getSuccessor(secondMatrix);
        copyMatrixes(secondMatrix, firstMatrix);

        //execute first operation. first we erase the previous result
        QSMatrix::multiplication(firstMatrix,secondMatrix, thirdMatrix);

        //then we saved the result
        QSMatrix *result = new QSMatrix(thirdMatrix->getSize());
        copyMatrixes(thirdMatrix, result);
        savedMatrixes.append(result);
        steps++;

        //then iterate
        while( (!QSMatrix::areEqual( thirdMatrix, firstMatrix)) && (steplimit > 0)  )
        {
            //then copy the result to the first matrix
            copyMatrixes(thirdMatrix, firstMatrix);

            //execute command
            QSMatrix::multiplication(firstMatrix,secondMatrix, thirdMatrix);
            //check if the result has already been founded (are we looping?)
            looping = false;
            for(int i=0; i<(savedMatrixes.size()-1); i++) //we don't have to check with the last (otherwise IS the fix point)
            {

                looping = QSMatrix::areEqual(thirdMatrix, savedMatrixes.at(i));
                if(looping)
                {
                    i = savedMatrixes.size()+1; //we exit from the cycle
                }
            }

            if(!looping)
            {
                //otherwise we save
                QSMatrix *result = new QSMatrix(thirdMatrix->getSize());
                copyMatrixes(thirdMatrix, result);
                savedMatrixes.append(result);
            }
            else
                steplimit = 0; //we exit from the cycle

            //and then iterate

            //decrement the limit
            steplimit--;
            steps++;
        }

        if(steps > maxsteps)
        {
            maxsteps = steps;
            qDebug() << "trovato nuovo record di passi: " << maxsteps;
        }

        //erase all results;
        qDeleteAll(savedMatrixes);
        savedMatrixes.clear();
    }

    cout << "il record per matrice di grandezza "<< firstMatrix->getSize() << " è stato: n. " << maxsteps << " passi" << endl;

}
