#include "qsmatrix.h"

QSMatrix::QSMatrix(int sizegiven)
{
    sizeMatrix = sizegiven;
    internalMatrix = new QVector<int>((sizeMatrix*sizeMatrix), E_NODEP );
}

QSMatrix::~QSMatrix()
{
    internalMatrix->clear();
    delete internalMatrix;
}


int QSMatrix::getElementAt(int row, int column)
{
    return internalMatrix->at( (sizeMatrix)*(row) + column );;
}

void QSMatrix::setElementAt(int row, int column, int element)
{
    internalMatrix->replace(  (sizeMatrix)*(row) + column , element);
}

int QSMatrix::getSize()
{
    return sizeMatrix;
}

void QSMatrix::reset()
{
    //reset the matrix with NODEP element
    for(int row=0; row< sizeMatrix; row++)
        for(int column=0; column< sizeMatrix; column++)
            setElementAt(row,column, E_NODEP);
}

void QSMatrix::multiplication(QSMatrix *a, QSMatrix *b, QSMatrix *result)
{
    int element = E_NODEP;

    for(int row = 0; row< (a->getSize()); row ++)
        for(int column = 0; column < (a->getSize()); column++)
        {
            //now we are creating element at position row,column
            //erase the sum
            element=E_NODEP;

            //start to sum
            for(int k = 0; k< a->getSize(); k++)
                element = addElement(element,
                           multElement(
                               a->getElementAt(row,k),
                               b->getElementAt(k,column)
                               )
                           );
            //setting the element in the matrix
            result->setElementAt(row,column , element );

        }
}

void QSMatrix::addition(QSMatrix *a, QSMatrix *b, QSMatrix *result)
{
    for(int row = 0; row<a->getSize(); row ++)
        for(int column = 0; column < a->getSize(); column++)
        {
            result->setElementAt(row,column,
                                            addElement(
                                                a->getElementAt(row,column),
                                                b->getElementAt(row,column)
                                                )
                                            );
        }
}

void QSMatrix::unions(QSMatrix *a, QSMatrix *b, QSMatrix *result)
{
    for(int row = 0; row< (a->getSize()); row ++)
        for(int column = 0; column < (a->getSize()); column++)
        {
            result->setElementAt(row,column,  fmax( a->getElementAt(row,column),
                                                   b->getElementAt(row,column) )  );
        }
}













//checks if two matrixes are equal or not
bool QSMatrix::areEqual(QSMatrix *a, QSMatrix *b)
{
    bool result = true;

    for(int row=0; row< a->getSize(); row++)
        for(int column=0; column< b->getSize(); column++)
        {
            if( a->getElementAt(row,column) != b->getElementAt(row,column) )
            {//ends the cicle and say false
                result = false;
                column = a->getSize() + 1;
                row = b->getSize() + 1;
            }
        }

    return result;
}

int QSMatrix::multElement(int a, int b)
{
    int result = E_NODEP;

    if(a == E_NODEP || b == E_NODEP)
        result = E_NODEP;
    else if (a == E_IDENTITY )
        result = b;
    else if (a == E_ADDED )
        if( b == E_MULT)
            result = b;
        else
            result = a;
    else //a è mult
        result = a;

//    qDebug() << a << " e " << b << "MULT" << result ;

    return result;
}
int QSMatrix::addElement(int a, int b)
{
    int result = E_NODEP;

    if(a == E_NODEP )
        result = b;
    else if(a == E_IDENTITY )
        if(b == E_NODEP )
            result = a;
        else if(b == E_IDENTITY )
            result = E_ADDED;
        else
            result = b;
    else if(a == E_ADDED )
        if(b == E_MULT )
            result = b;
        else
            result = a;
    else
        result = a;

//    qDebug() << a << " e " << b << "danno" << result;

    return result;
}


void QSMatrix::getSuccessor(QSMatrix *a)
{
//    cout << "voglio calcolare il successore di " << a->toString() << endl;

    //take the element at bottom right
    int size = a->getSize();
    int element = a->getElementAt(size-1,size-1);

    if(element < E_MULT)
        a->setElementAt(size-1,size-1,element+1);
    else if(a->getSize() > 1)
    {
        a->setElementAt(size-1,size-1,  E_NODEP );
        getSuccessorWIndex(a, size-1, size-2);
    }


}

void QSMatrix::getSuccessorWIndex(QSMatrix *a, int row, int column)
{
    int element = a->getElementAt(row,column);
    int size = a->getSize();

    if(element < E_MULT)
        a->setElementAt(row,column,element+1);
    else if(column > 0)
    {
        a->setElementAt(row,column,  E_NODEP );
        getSuccessorWIndex(a, row, column-1);
    }
    else if(row > 0 )
    {
        a->setElementAt(row,column,  E_NODEP );
        getSuccessorWIndex(a, row-1, size-1);
    }
}


string QSMatrix::toString()
{
    stringstream result;

    for(int i=0; i<(sizeMatrix*sizeMatrix); i++)
        result << internalMatrix->at(i);

    return result.str();
}
